# -*- coding: utf-8 -*-

from __future__ import unicode_literals


class Grid(object):
    def __init__(self, rows):
        if not (isinstance(rows, list) or isinstance(rows, tuple)):
            raise TypeError('Grid elements attribute '
                            'must be a list or tuple')
        if not all([isinstance(r, Row) for r in rows]):
            raise TypeError('All elements of Grid '
                            'must be Row instances')
        self.type = 'grid'
        self.rows = rows


class Row(object):
    def __init__(self, columns):
        if not (isinstance(columns, list) or isinstance(columns, tuple)):
            raise TypeError('Row elements attribute '
                            'must be a list or tuple')
        if not all([isinstance(c, Column) for c in columns]):
            raise TypeError('All elements of Row '
                            'must be Column instances')
        self.type = 'row'
        self.columns = columns


class Column(object):
    def __init__(self, elements, width=12):
        if not (isinstance(elements, list) or isinstance(elements, tuple)):
            raise TypeError('Column elements attribute '
                            'must be a list or tuple')
        if any([isinstance(e, Column) for e in elements]):
            raise TypeError('Elements of Column '
                            'cannot be Column instances')
        if width not in range(1, 13):
            raise ValueError('Column width must be between 1 and 12')

        self.type = 'column'
        self.elements = elements
        self.width = width



# -*- coding: utf-8 -*-

from __future__ import unicode_literals


class Widget(object):
    def __init__(self, id=None, title=None, description=None,
                 context=None, template=None):
        if context:
            if not (isinstance(context, list) or isinstance(context, tuple)):
                raise AttributeError('Widget context attribute '
                                     'must be a list or tuple')
            if not all([(isinstance(e, WidgetGroup) or
                         isinstance(e, WidgetItem)) for e in Widget]):
                raise ValueError('Elements of Widget context must be '
                                 'WidgetGroup or WidgetItem instances')
            self.context = context

        if id:
            self.id = id
        if title:
            self.title = title
        if description:
            self.description = description
        if template:
            self.template = template

        self.type = 'widget'


class WidgetGroup(object):
    AS_TABLE = 'table'
    AS_LIST = 'list'

    def __init__(self, id, name, items, display='table', classes=''):
        self.type = 'group'
        self.id = id
        self.name = name
        self.display = display
        self.items = items
        self.classes = classes


class WidgetItem(object):
    def __init__(self, id, name, value, is_chart=False):
        self.type = 'item'
        self.id = id
        self.name = name
        self.value = value
        self.is_chart = is_chart
